package cologne.eck.note_pea;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.Window;
import java.util.ArrayList;

import javax.accessibility.AccessibleContext;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.text.DefaultStyledDocument;

import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.IconManager;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Comparator;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.Zeroizer;




@SuppressWarnings("serial")
public class TabComponent extends JScrollPane 
		implements ActionListener, // for close button of tab
		MouseListener { // for CopyCutPastePopoup
	
	/**
	 * The JTabbedPane where this instance is added to
	 */
	private final JTabbedPane tabPane;
	
	/**
	 * The text pane associated with this instance 
	 */
	private JTextPane textPane;
	
	/**
	 * The name of the file containing the encrypted content
	 * and associated with this instance
	 */
	private String fileName;


	/**
	 * TabComponent is a JScrollPane which is used for the JTabbedPane
	 * of the Notebook PEA. Each TabComponent is associated with a 
	 * file name of an encrypted text file and a JTextPane to display 
	 * the file content. 
	 * If the tab is selected, the title of the tab contains a pea icon 
	 * and the background is green, otherwise the background is gray. 
	 * 
	 * @param tabPane	the JTabbedPane that contains this TabComponent
	 * @param fileName	the name of the associated file 
	 */
	public TabComponent(final JTabbedPane tabPane, String fileName) {
		
        if (tabPane == null) {
            throw new NullPointerException("TabComponent: JTabbedPane is null");
        }
        if (fileName == null) {
        	// do not allow null file name in constructor
        	throw new NullPointerException("TabComponent: file name is null");
        }
        this.tabPane = tabPane;
        this.fileName = fileName;
        this.setBorder(BorderFactory.createEmptyBorder());
        
        int fontSize = PswDialogView.getFontsize();
        
        // create the JTextPane:
        textPane = new JTextPane();	
        textPane.setName(this.fileName);
		textPane.setEditable(true);
		textPane.setBackground(new Color(231, 231, 231) );
		textPane.setDragEnabled(true);
		textPane.setToolTipText("CTRL + ...  R (undo), W (redo), X (cut), C (copy), V (paste)");
		// set the JTextPane as component for this instance: 
		this.setViewportView(textPane);
		
		// add this instance of TabComponent to the tabPane:
		tabPane.addTab(
				this.fileName,	
				null,
				this, 
				null);
		//tabPane.setMnemonicAt(0, KeyEvent.VK_1);
		
		// set the close button for the tab and LIGHT_GRAY as background color
		// to indicate that the file is untouched
		JPanel tabTitleDisplay = new JPanel();
		JLabel titleLabel = new JLabel();
		titleLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		// cut the file name and display the last part with leading ellipsis (…)
		String labelText = null;
		int len = this.fileName.length();
		if (len > 16 + 3) { // length + ellipsis
			labelText = "..." + this.fileName.substring(len - 16,len);
		} else {
			labelText = this.fileName;
		}
		titleLabel.setText(labelText);
		JLabel iconLabel = new JLabel();
		//iconLabel.setIcon(IconManager.loadScaledIcon("locked.png", "closed lock", 20, 20));
		iconLabel.setIcon(IconManager.loadIcon("locked.png", PeaProperties.getBundle().getString("closed_padlock"), 
				null, PeaProperties.getBundle().getString("encrypted_indicator") ) );
		tabTitleDisplay.add( iconLabel);
		tabTitleDisplay.setBackground(Color.LIGHT_GRAY);
		titleLabel.setBackground(Color.LIGHT_GRAY);
		tabTitleDisplay.add( titleLabel);
		JButton closeButton = new JButton("x");
		closeButton.addActionListener(this);
		tabTitleDisplay.add(closeButton);
		tabPane.setTabComponentAt(tabPane.indexOfComponent(this), tabTitleDisplay);
	}

	
	/**
	 * Display text as DefaultStyledDocument in the TextPane
	 * of this instance. 
	 * 
	 * @param plainBytes	the text to display as an array of bytes
	 * @param textPane		the TextPane to display
	 */
	private final void displayText(byte[] plainBytes, JTextPane textPane) {
		
		Object obj = Converter.deserialize( plainBytes);		
		
		if (! (obj instanceof DefaultStyledDocument) ) { // wrong psw (malformed file) not all wrong files
			System.err.println("unsuitable document, not a styled document");
			System.err.println("This file is not a valid rft file. This program can not open this file.");
			
			PswDialogView.setMessage("unsuitable_rtf_file", true);
			
			JOptionPane.showMessageDialog(PswDialogView.getView(),
					PeaProperties.getBundle().getString("unsuitable_rtf_file"),
					PeaProperties.getBundle().getString("error"),
				    JOptionPane.ERROR_MESSAGE);

			textPane.setText(new String(plainBytes,PeaProperties.getCharset()));

		} else {
			DefaultStyledDocument doc = (DefaultStyledDocument) obj;
			textPane.setStyledDocument(doc);	
		}						
		Zeroizer.zero(plainBytes);
	}
	
	
	/**
	 * Read the associated file, decrypt the content 
	 * and set the decrypted content as text in the text pane. 
	 * Possible errors are file == null (no dialog displayed), 
	 * file access (dialog), different salt (dialog), and 
	 * decryption/password failure (dialog). 
	 *  
	 * @return	an error message if the decryption failed
	 * 			or null if the decryption was successful
	 */
	protected String setText() {
		
		//System.out.println("TabComp setText: " + this.fileName);
		
		String errorMessage = null;
		
		if (this.fileName == null) {
			// file name was set null before because of decryption error
			System.err.println("TabComponent: no file name " );
			errorMessage = PeaProperties.getBundle().getString("no_valid_file_found");
/*			JOptionPane.showMessageDialog(this,
				    PswDialogBase.getBundle().getString("no_valid_file_found")
				    + "\n - - - - ",
				    "Decryption failed",
				    JOptionPane.ERROR_MESSAGE);			*/
			return errorMessage;
		}		
		
		try {
			// set the wait cursor: 
			((Window) PeaControl.getDialog().getLockFrame()).setCursor(CursorManager.getWaitCursor());


	
			// check file (existence, access...):
			String fileName = this.fileName;//textPane.getName();
			byte[] cipherBytes = ReadResources.readExternFile(fileName);
			if (cipherBytes == null) {			
				// reset the cursor:
				((Window) PeaControl.getDialog().getLockFrame()).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

				System.err.println("TabComponent: Access failed: " + fileName);
				errorMessage = PeaProperties.getBundle().getString("no_access");
				JOptionPane.showMessageDialog(this,
						PeaProperties.getBundle().getString("no_access")
					    + "\n" + fileName,
					    PeaProperties.getBundle().getString("error") + ": " + PeaProperties.getBundle().getString("decryption"),
					    JOptionPane.ERROR_MESSAGE);			
				return errorMessage;
			}
			
			// check salt:
			byte[] checkSalt = new byte[KeyDerivation.getSaltSize()];
			System.arraycopy(cipherBytes, cipherBytes.length - Attachments.getFileIdentifierSize() 
					- KeyDerivation.getSaltSize(), 
					checkSalt, 0, KeyDerivation.getSaltSize());
			if (Comparator.compare(checkSalt, KeyDerivation.getSalt()) == false){
				// reset the cursor:
				((Window) PeaControl.getDialog().getLockFrame()).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

				System.err.println("TabComponent: Salt failed: " + fileName);
				errorMessage = PeaProperties.getBundle().getString("salt_differs_from_first_selected_file");
				JOptionPane.showMessageDialog(this,
						PeaProperties.getBundle().getString("salt_differs_from_first_selected_file")
					    + "\n" + fileName,
					    PeaProperties.getBundle().getString("error") + ": " + PeaProperties.getBundle().getString("decryption"),
					    JOptionPane.ERROR_MESSAGE);			
				return errorMessage;
			}
			
			// check password and mac
			byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
			if (plainBytes == null){
				// decryption failed
				
				// reset the cursor:
				((Window) PeaControl.getDialog().getLockFrame()).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

				errorMessage = CipherStuff.getErrorMessage();
				if (errorMessage == null) {
					errorMessage = "decryption error";
				}
				
				System.err.println("TabComponent: Decryption failed: " + errorMessage + ", " + fileName);
				JOptionPane.showMessageDialog(this,
						PeaProperties.getBundle().getString(errorMessage)				   
					    + "\n" + fileName,
					    PeaProperties.getBundle().getString("error") + ": " + PeaProperties.getBundle().getString("decryption"),
					    JOptionPane.ERROR_MESSAGE);
				return errorMessage;
			}
			
			displayText(plainBytes, textPane);
			LockFrameEditor.setDocChangeUnsaved(false);
		} finally {
			// reset the cursor:
			((Window) PeaControl.getDialog().getLockFrame()).setCursor(CursorManager.getDefaultCursor());
		}
		return errorMessage;
	}
	/**
	 * Set the title of a TabComponent and a close button. 
	 * If the tab is focused, the title contains a pea icon 
	 * and the background is green, otherwise the background
	 * is gray. 
	 * 
	 * @param iconified		true if the tab should display an icon
	 * 						(should be marked as selected/focused)
	 */
	protected void setTabTitle(boolean iconified) {
		
		//System.out.println("TabComp setTabTitle: " + this.fileName + ", focus: " + iconified);
		
		if (tabPane.indexOfComponent(this) == -1){
			System.err.println("set tab title failed: Tab does not exist");
			return;
		}

		JPanel tabTitleDisplay = new JPanel();
		JLabel titleLabel = new JLabel();
		
		if (iconified == true) {
			
			// cut the file name and display the last part with leading ellipsis (…)
			String labelText = null;
			int len = this.fileName.length();
			if (len > 80 + 3) {
				labelText = "..." + this.fileName.substring(len - 80,len);
			} else {
				labelText = this.fileName;
			}
			titleLabel.setText(labelText);
			titleLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 
					PswDialogView.getFontsize()));
			JLabel iconLabel = new JLabel();
			Image img = PswDialogView.getImage().getScaledInstance(20,20,Image.SCALE_DEFAULT);
			iconLabel.setIcon(new ImageIcon(img));
			tabTitleDisplay.add( iconLabel);
			tabTitleDisplay.setBackground(new Color(229, 255, 231));
			// maybe not necessary: 
			tabPane.setBackgroundAt(tabPane.indexOfComponent(this), new Color(229, 255, 231));
			titleLabel.setBackground(new Color(229, 255, 231));
			
		} else {
			// cut the file name and display the last part with leading ellipsis (…)
			String labelText = null;
			int len = this.fileName.length();
			if (len > 16 + 3) {
				labelText = "..." + this.fileName.substring(len - 16,len);
			} else {
				labelText = this.fileName;
			}
			titleLabel.setText(labelText);
			titleLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 
					PswDialogView.getFontsize()));
			
			JLabel iconLabel = new JLabel();
			//iconLabel.setIcon(IconManager.loadScaledIcon("locked.png", "closed lock", 20, 20));
			iconLabel.setIcon(IconManager.loadIcon("locked.png", PeaProperties.getBundle().getString("closed_padlock"), 
					null, PeaProperties.getBundle().getString("encrypted_indicator") ) );
			tabTitleDisplay.add( iconLabel);
			
			//titleLabel.setPreferredSize(new Dimension(200, 30) );
			tabTitleDisplay.setBackground(new Color(216, 216, 216));//Color.LIGHT_GRAY);
			// maybe not necessary: 
			tabPane.setBackgroundAt(tabPane.indexOfComponent(this), new Color(216, 216, 216));
			titleLabel.setBackground(new Color(216, 216, 216));//Color.LIGHT_GRAY);
		}
		tabTitleDisplay.add( titleLabel);//new JLabel(this.fileName ));
		JButton closeButton = new JButton("x");
		closeButton.addActionListener(this);
		tabTitleDisplay.add(closeButton);
		tabPane.setTabComponentAt(tabPane.indexOfComponent(this), tabTitleDisplay);
	}



	@Override
	public void actionPerformed(ActionEvent ape) {

		// the tab to close = this instance 
		int closingTabIndex = tabPane.indexOfComponent(this);
		// the actually selected tab in the tabPane
		int selectedTabIndex = tabPane.indexOfComponent(tabPane.getSelectedComponent());
		if (closingTabIndex == selectedTabIndex) {
			// handle unsaved changes:
			( (LockFrameEditor) PeaControl.getDialog().getLockFrame()).checkUnsavedTextChanges();
		}
		// ask to remember file name: 
		//PswDialogBase.setEncryptedFileName(this.fileName);
		PathFileManager.pathFileCheck(this.fileName); // checks encryptedFileName
		
		// remove the file name from the list of available file names
		ArrayList<String> list = PeaControl.getAvailableFileNames();
		if (list.contains(fileName)) {
			list.remove(fileName);
			PeaControl.setAvailableFileNames(list);
		}
		
		// remove the this tab
		if (tabPane.indexOfComponent(this) != -1) {
			// first set previously selected tab selected again:
			TabComponent lastFocusTab = LockFrameEditor.getCurrentTab();
			if (lastFocusTab != null && (tabPane.indexOfComponent(lastFocusTab) != -1)) {
				tabPane.setSelectedComponent(LockFrameEditor.getCurrentTab());
			}
			if (tabPane.indexOfComponent(this) != -1
					&& tabPane.getTabCount() > 0) {
				try { // if no valid tab is left, this will throw an IndexOutOfBoundExcetion
					tabPane.remove(tabPane.getComponentAt(closingTabIndex));
				} catch (IndexOutOfBoundsException e) {
					System.err.println(e.toString() + ", closing tab failed: " + e.getMessage());
				}
			}
		}
	}
	
	@Override
	public void mouseClicked(MouseEvent mce) { // cut-copy-paste

		if(mce.getModifiers() == InputEvent.BUTTON3_MASK){ //rechte Maustaste
			Component invoker = mce.getComponent();
			CopyCutPastePopup popup = CopyCutPastePopup.getInstance(invoker);
			
			if (popup.isVisible()){

				popup.setVisible(false);
			} else {

				try {
					popup.setLocation(mce.getLocationOnScreen());				
					popup.setVisible(true);				
				} catch (NullPointerException npe) {
					System.err.println("Popup in EditorLockFrame: NullPointer");
				}	
			}
		} else { // left mouse button
			
		}
	}
	@Override
	public void mouseEntered(MouseEvent arg0) {}
	@Override
	public void mouseExited(MouseEvent arg0) {}
	@Override
	public void mousePressed(MouseEvent arg0) {}
	@Override
	public void mouseReleased(MouseEvent arg0) {}

	/**
	 * Get the file name associated with this TabComponent
	 * 
	 * @return  the file name	
	 */
	public String getFileName() {
		return this.fileName;
	}
	/**
	 * Set the file name associated with this TabComponent
	 * 
	 * @param  the new file name
	 */
	public void setFileName(String newFileName) {
		this.fileName = newFileName;
	}
	
	/**
	 * Get the JTextPane, displaying the decrypted file content, 
	 * associated with this instance of TabCompoent
	 * 
	 * @return	the JTextPane associated with this TabComponent
	 */
	protected JTextPane getTextPane(){
		return textPane;
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleTabComponent();
        }
        return accessibleContext;
    }

    protected class AccessibleTabComponent extends AccessibleJScrollPane {
        //Inherit everything, override nothing.
    }
}
